#!/usr/bin/perl -n -i~s
#
# clean up LyX files with iso8859-1 char set characters so that it
# can be sent to people who want to use plain ASCII.
# 
# This is done by replacing iso8859-1 extended characters with LaTeX 
# macros for those characters, e.g. accepted characters, math 
# symbols etc.
#
# Just run this program on .lyx files and they will be modified in
# place. A backup will be make by appending the suffix ~s

# First set up a table of accented characters 
%accents = (
	"\300" => [ "A", '`' ],
	"\301" => [ "A", "'" ],
	"\302" => [ "A", "^" ],
	"\303" => [ "A", "~" ],
	"\304" => [ "A", '"' ],
	"\307" => [ "C", 'c' ],
	"\310" => [ "E", '`' ],
	"\311" => [ "E", "'" ],
	"\312" => [ "E", "^" ],
	"\313" => [ "E", '"' ],
	"\310" => [ "I", '`' ],
	"\311" => [ "I", "'" ],
	"\312" => [ "I", "^" ],
	"\313" => [ "I", '"' ],
	"\322" => [ "O", '`' ],
	"\323" => [ "O", "'" ],
	"\324" => [ "O", "^" ],
	"\325" => [ "O", "~" ],
	"\326" => [ "O", '"' ],
	"\331" => [ "U", '`' ],
	"\332" => [ "U", "'" ],
	"\333" => [ "U", "^" ],
	"\334" => [ "U", '"' ],
	"\340" => [ "a", '`' ],
	"\341" => [ "a", "'" ],
	"\342" => [ "a", "^" ],
	"\343" => [ "a", "~" ],
	"\344" => [ "a", '"' ],
	"\347" => [ "c", 'c' ],
	"\350" => [ "e", '`' ],
	"\351" => [ "e", "'" ],
	"\352" => [ "e", "^" ],
	"\353" => [ "e", '"' ],
	"\354" => [ "i", '`' ],
	"\355" => [ "i", "'" ],
	"\356" => [ "i", "^" ],
	"\357" => [ "i", '"' ],
	"\362" => [ "o", '`' ],
	"\363" => [ "o", "'" ],
	"\364" => [ "o", "^" ],
	"\365" => [ "o", "~" ],
	"\366" => [ "o", '"' ],
	"\371" => [ "u", '`' ],
	"\372" => [ "u", "'" ],
	"\373" => [ "u", "^" ],
	"\374" => [ "u", '"' ],
);

# clean no-break spaces (just zap them).  These seem to be 
# introduced sometimes by copy-paste operation from other apps.  We
# don't want them in out LyX document.
s/\240//g;

# degree symbol
s/([\d\.]*)?\260/&degree_symbol($1)/eg;

# vulgar fractions
s/\274/&vulgar_fraction(1,4)/eg;
s/\275/&vulgar_fraction(1,2)/eg;
s/\276/&vulgar_fraction(3,2)/eg;

# accented charactcers
# since there are more accented characters than characters on a typical
# line of LyX markup, we'll just test each character on the line to see
# if it is accented.
for(my $i=0; $i<length($_) - 1; $i++) {
	my $c = substr($_, $i, 1);
	if (defined($accents{$c}) ) {
		substr($_, $i, 1) = accent_char($c);
	}
}

# print a warning if there are un-handled non-ASCII characters.
if ( /([\200-\377])/ ) {
	my $c = $1;
	$c = "\"" 
		. $c 
		. "\"" 
		. " (" . ord($c) 
		. " dec / " 
		. sprintf("%o", ord($c)) 
		. " octal)";
	print STDERR "scrub WARNING: unhandled non-ascii character(s), $c at line $. of $ARGV\n";
}

print;


sub degree_symbol {
	my $num = shift || "";
	"
\\begin_inset
Formula \$$num^\{\\circ\}\$
\\end_inset
";
}

sub vulgar_fraction {
	my ($numerator, $denominator) = (@_);
	"
\\begin_inset
Formula \$\\frac\{$numerator\}\{$denominator\}\$
\\end_inset
";
}

sub accent_char {
	my ($c) = $_[0] || die "accent_char() : expected char, didn't get one";
	my ($char, $ac) = @{$accents{$c}};
	
	"
\\begin_inset ERT
status open
\\begin_layout Standard
\\backslash
$ac\{$char\}
\\end_layout
\\end_inset
";
}
